package com.bwie.chat.component;

import com.alibaba.fastjson.JSON;
import com.bwie.chat.config.ThreadPoolExecutorConfig;
import com.bwie.chat.vo.Message;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 注意在websocket通信中只能传string
 */
@Component
@ServerEndpoint("/socket/{username}")
public class WebSocketServer {
    //静态变量,用于记录当前在线连接数,应该把他设计为线程安全
    private static AtomicInteger onlineNum=new AtomicInteger();
    private final ThreadPoolExecutorConfig thread;
//    存储当前对象
    public static final Map<String, Session> sessionMap = new ConcurrentHashMap<>();

    public WebSocketServer(ThreadPoolExecutorConfig thread) {
        this.thread = thread;
    }


    public static void addOnlineCount(){
        onlineNum.incrementAndGet();
    }

    public static void subOnlineCount(){
        onlineNum.decrementAndGet();
    }

    public static AtomicInteger getOnlineNum(){
        return onlineNum;
    }

//    建立连接
    /***
     * 1.把登录用户存到sessionMap中
     * 2.发送给所有人当前登录人员信息
     */
    @OnOpen
    public void onOpen(Session session, @PathParam("username") String username) {
        CompletableFuture.supplyAsync(()->{
            System.out.println("当前用户名=="+username);
            sessionMap.put(username, session);
            addOnlineCount();
            System.out.println(username+"成功连接,当前人数为"+onlineNum);
            // 发送登录人员消息给所有的客户端
            sendAllMessage(setUserList());
            return "打开连接";
                },thread.threadPoolExecutor());


    }

    //关闭连接
    /**
     * 1.把登出的用户从sessionMap中剃除
     * 2.发送给所有人当前登录人员信息
     */
    @OnClose
    public void onClose(@PathParam("username") String username) {
        CompletableFuture.supplyAsync(()->{
        sessionMap.remove(username);
        subOnlineCount();
        System.out.println(username+"断开连接,当前人数为"+onlineNum);
        sendAllMessage(setUserList());
            return "关闭连接";
        },thread.threadPoolExecutor());
    }

    /**
     * 接收处理客户端发来的数据
     */
    @OnMessage
    public void onMessage(String message) {
//        解析消息为java对象
        Message msg = JSON.parseObject(message, Message.class);
        if(StringUtils.isEmpty(msg.getTo())){
            sendAllMessage(JSON.toJSONString(msg));
        }else{
            Session sessionTo = sessionMap.get(msg.getTo());
            sendMessage(message,sessionTo);
        }
    }

    @OnError
    public void onError(Session session, Throwable error) {
        System.out.println("发生错误");
        error.printStackTrace();
    }

    private String setUserList(){
        ArrayList<String> list = new ArrayList<>();
        for(String key:sessionMap.keySet()){
            list.add(key);
        }
        Message message = new Message();
        message.setUserNames(list);
        return JSON.toJSONString(message);
    }



//    服务端发送消息给指定客户端
    private void sendMessage(String message, Session toSession) {
        CompletableFuture.supplyAsync(()->{

                try {
                    synchronized (toSession){
                    toSession.getBasicRemote().sendText(message);
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }


                    return "指定发送";
        },thread.threadPoolExecutor());

    }

//   服务端发送消息给所有客户端
    private void sendAllMessage(String message) {
        CompletableFuture.supplyAsync(()->{
            for (Session session : sessionMap.values()) {
                try {
                    synchronized (session) {
                        session.getBasicRemote().sendText(message);
                    }
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
            }
            return "系统群发";
        },thread.threadPoolExecutor());

    }
}
